import base64
import glob
import os
import shutil


def ensure_directory(d):
    d = os.path.abspath(os.path.expanduser(d))
    if not os.path.exists(d):
        os.makedirs(d)
    return d


def chunk(lst, n):
    for i in range(0, len(lst), n):
        yield lst[i:i + n]


def find_module(module_dirs, module_name):

    '''
    Finds a module file path in the module_dirs with the name module_name.

    Returns a file path.
    '''

    # Find the module in module_dirs
    for d in module_dirs:
        module = os.path.join(d, f'{module_name}.py')
        if os.path.exists(module):
            break
        else:
            module = None

    return module


def encode_module(module_dirs, module_name):

    with open(find_module(module_dirs, module_name), 'rb') as f:
        return base64.b64encode(f.read()).decode()


def read_module(module_dirs, module_name):

    with open(find_module(module_dirs, module_name), 'rb') as f:
        return f.read()


def clean_up_ftl_cache():
    cache = os.path.abspath(os.path.expanduser("~/.ftl"))
    if os.path.exists(cache) and os.path.isdir(cache) and ".ftl" in cache:
        shutil.rmtree(cache)


def clean_up_tmp():
    for d in glob.glob('/tmp/ftl-*'):
        if os.path.exists(d) and os.path.isdir(d) and 'tmp' in d and 'ftl' in d:
            shutil.rmtree(d)



"""Top-level package for Faster Than Light."""

__author__ = """Ben Thomasson"""
__email__ = 'benthomasson@gmail.com'
__version__ = '0.1.2'

from .module import run_module, run_ftl_module, close_gate
from .inventory import load_inventory

__all__ = ['run_module', 'run_ftl_module', 'load_inventory', 'close_gate']




import asyncio
import json
import os
import sys
import tempfile
import base64
import ftl_gate
import importlib.resources
import shutil


async def connect_stdin_stdout():
    loop = asyncio.get_event_loop()
    reader = asyncio.StreamReader()
    protocol = asyncio.StreamReaderProtocol(reader)
    await loop.connect_read_pipe(lambda: protocol, sys.stdin)
    w_transport, w_protocol = await loop.connect_write_pipe(asyncio.streams.FlowControlMixin, sys.stdout)
    writer = asyncio.StreamWriter(w_transport, w_protocol, reader, loop)
    return reader, writer


async def read_message(reader):

    while True:

        # Messages are Length Value
        # Length is a 8 byte field in hexadecimal
        # Value is a length byte field

        value = b''
        length_hexadecimal = b'0'
        length = 0

        # Read length
        length_hexadecimal = await reader.read(8)
        # If the length_hexadecimal is None then the channel was closed
        if not length_hexadecimal:
            return None

        # Read value
        if length_hexadecimal.strip():
            length = int(length_hexadecimal.strip(), 16)
            if length == 0:
                continue
            # If value is a new-line try the read operation again
            while True:
                value = await reader.read(length)
                # If the value is None then the channel was closed
                if not value:
                    return None
                value = value.strip()
                # Keep reading until we get a value
                # This is useful for manual debugging
                # Run with python __main__.py
                # Enter: 0000000d
                # Enter: ["Hello", {}]
                # Response should be  0000000d["Hello", {}]
                if value:
                    try:
                        return json.loads(value)
                    except BaseException:
                        print(value)
                        raise
                else:
                    continue


def send_message(writer, msg_type, data):

    # A message has a Length[Type, Data] format.
    # The first 8 bytes are the length in hexadecimal
    # The next Length bytes are JSON encoded data.
    # The JSON encoded data is a pair where the first
    # item is the message type and the second
    # item is the data.
    message = json.dumps([msg_type, data]).encode()
    assert len(message) < 16**8, f'Message {msg_type} is too big.  Break up messages into less than 16**8 bytes'
    writer.write('{:08x}'.format(len(message)).encode())
    writer.write(message)


async def check_output(cmd):
    proc = await asyncio.create_subprocess_shell(
        cmd,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE)

    stdout, stderr = await proc.communicate()
    return stdout, stderr


async def gate_run_module(writer, module_name, module=None):
    tempdir = tempfile.mkdtemp(prefix="ftl-module")
    try:
        module_file = os.path.join(tempdir, module_name)
        if module is not None:
            with open(module_file, 'wb') as f:
                f.write(base64.b64decode(module))
        else:
            modules = importlib.resources.files(ftl_gate)
            with open(module_file, 'wb') as f2:
                f2.write(importlib.resources.read_binary(ftl_gate, module_name))
        args = os.path.join(tempdir, 'args')
        with open(args, 'w') as f:
            f.write('some args')
        stdout, stderr = await check_output(f'{sys.executable} {module_file} {args}')
        send_message(writer, 'ModuleResult', dict(stdout=stdout.decode(),
                                                  stderr=stderr.decode()))
    finally:
        shutil.rmtree(tempdir)


async def run_ftl_module(writer, module_name, module):

    module_compiled = compile(base64.b64decode(module), module_name, 'exec')

    globals = {'__file__': module_name}
    locals = {}

    exec(module_compiled, globals, locals)
    result = await locals['main']()
    send_message(writer, 'FTLModuleResult', dict(result=result))


async def main(args):

    reader, writer = await connect_stdin_stdout()

    while True:

        try:
            msg_type, data = await read_message(reader)
            if msg_type == 'Hello':
                send_message(writer, msg_type, data)
            elif msg_type == 'Module':
                await gate_run_module(writer, **data)
            elif msg_type == 'FTLModule':
                await run_ftl_module(writer, **data)
            elif msg_type == 'Shutdown':
                return
            else:
                send_message(writer, 'Error', dict(message=f'Unknown message type {msg_type}'))
        except BaseException as e:
            send_message(writer, 'GateSystemError', dict(message=f'Exception {e}'))
            return 1


if __name__ == "__main__":
    asyncio.run(main(sys.argv[1:]))





import os
import pytest
from pprint import pprint
from faster_than_light.inventory import load_inventory
from faster_than_light.module import check_output
from faster_than_light.module import run_module, run_ftl_module
from faster_than_light.util import clean_up_ftl_cache, clean_up_tmp

HERE = os.path.dirname(os.path.abspath(__file__))


@pytest.mark.asyncio
async def test_checkoutput():
    os.chdir(HERE)
    output = await check_output('ping')
    print(output)
    assert output
    clean_up_ftl_cache()
    clean_up_tmp()


@pytest.mark.asyncio
async def test_run_module_timetest():
    os.chdir(HERE)
    output = await run_module(load_inventory('inventory.yml'), ['modules'], 'timetest')
    pprint(output)
    assert output['localhost']
    clean_up_ftl_cache()
    clean_up_tmp()


@pytest.mark.asyncio
async def test_run_module_argtest():
    os.chdir(HERE)
    output = await run_module(load_inventory('inventory.yml'), ['modules'], 'argtest')
    pprint(output)
    assert output['localhost']
    clean_up_ftl_cache()
    clean_up_tmp()


@pytest.mark.asyncio
async def test_run_module_argtest_remote():
    os.chdir(HERE)
    output = await run_module(load_inventory('inventory2.yml'), ['modules'], 'argtest')
    pprint(output)
    assert output['localhost']
    clean_up_ftl_cache()
    clean_up_tmp()


@pytest.mark.asyncio
async def test_run_ftl_module():
    os.chdir(HERE)
    output = await run_ftl_module(load_inventory('inventory.yml'),
                                  ['ftl_modules'],
                                  'argtest')
    pprint(output)
    assert output['localhost']
    assert output['localhost'] == {'args': (), 'kwargs': {}}
    clean_up_ftl_cache()
    clean_up_tmp()



#!/usr/bin/python

import os
import sys
import json
import glob

args = sys.argv
with open(sys.argv[0]) as f:
    executable = f.read()
with open(sys.argv[1]) as f:
    more_args = f.read()
files = glob.glob(os.path.join(os.path.dirname(sys.argv[0]), '*'))

print(json.dumps({
    "args" : args,
    "executable": executable,
    "more_args": more_args,
    "files": files
}))







"""
Usage:
    ftl [options]

Options:
    -h, --help                  Show this page
    -f=<f>, --ftl-module=<f>    FTL module
    -m=<m>, --module=<m>        Module
    -M=<M>, --module-dir=<M>    Module directory
    -i=<i>, --inventory=<i>     Inventory
    -r=<r>, --requirements      Python requirements
    -a=<a>, --args=<a>          Module arguments
    --debug                     Show debug logging
    --verbose                   Show verbose logging
"""
import asyncio
from docopt import docopt
import logging
import sys
from .module import run_module
from .module import run_ftl_module
from .inventory import load_inventory

logger = logging.getLogger('cli')


def parse_module_args(args):
    key_value_pairs = args.split(' ')
    return dict([tuple(i.split('=')) for i in key_value_pairs])


async def main(args=None):
    if args is None:
        args = sys.argv[1:]
    parsed_args = docopt(__doc__, args)
    if parsed_args['--debug']:
        logging.basicConfig(level=logging.DEBUG)
    elif parsed_args['--verbose']:
        logging.basicConfig(level=logging.INFO)
    else:
        logging.basicConfig(level=logging.WARNING)

    dependencies = None
    if parsed_args['--requirements']:
        with open(parsed_args['--requirements']) as f:
            dependencies = [x for x in f.read().splitlines() if x]

    if parsed_args['--module']:
        output = await run_module(load_inventory(parsed_args['--inventory']),
                                  [parsed_args['--module-dir']],
                                  parsed_args['--module'],
                                  modules=[parsed_args['--module']],
                                  module_args=parse_module_args(parsed_args['--args']),
                                  dependencies=dependencies)
        print(output)
    elif parsed_args['--ftl-module']:
        output = await run_ftl_module(load_inventory(parsed_args['--inventory']),
                                      [parsed_args['--module-dir']],
                                      parsed_args['--ftl-module'])
        print(output)
    return 0


def entry_point():
    asyncio.run(main(sys.argv[1:]))





import asyncio
import os
import pytest
import base64
from pprint import pprint

from faster_than_light.message import read_message, send_message
from faster_than_light.gate import build_ftl_gate
from faster_than_light.module import run_module_on_host, find_module
from faster_than_light.util import clean_up_ftl_cache, clean_up_tmp

HERE = os.path.dirname(os.path.abspath(__file__))


@pytest.mark.asyncio
async def test_read_message():
    os.chdir(HERE)
    ftl_gate = build_ftl_gate()
    proc = await asyncio.create_subprocess_shell(
        ftl_gate,
        stdin=asyncio.subprocess.PIPE,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE)

    try:
        proc.stdin.write(b'0000000d["Hello", {}]')
        message = await read_message(proc.stdout)
        assert message[0] == "Hello"
        assert message[1] == {}
    except Exception:
        print((await proc.stderr.read()).decode())
        raise
    finally:
        send_message(proc.stdin, 'Shutdown', {})
        await proc.wait()
        os.unlink(ftl_gate)
        clean_up_ftl_cache()
        clean_up_tmp()

@pytest.mark.asyncio
async def test_build_ftl_gate():
    os.chdir(HERE)
    ftl_gate = build_ftl_gate()
    proc = await asyncio.create_subprocess_shell(
        ftl_gate,
        stdin=asyncio.subprocess.PIPE,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE)

    try:
        send_message(proc.stdin, 'Hello', {})
        message = await read_message(proc.stdout)
        assert message[0] == "Hello"
        assert message[1] == {}
    finally:
        send_message(proc.stdin, 'Shutdown', {})
        await proc.wait()
        os.unlink(ftl_gate)
        clean_up_ftl_cache()
        clean_up_tmp()


@pytest.mark.asyncio
async def test_run_module():
    os.chdir(HERE)
    ftl_gate = build_ftl_gate()
    proc = await asyncio.create_subprocess_shell(
        ftl_gate,
        stdin=asyncio.subprocess.PIPE,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE)

    try:
        send_message(proc.stdin, 'Hello', {})
        message = await read_message(proc.stdout)
        assert message[0] == "Hello"
        assert message[1] == {}

        with open(find_module(['modules'], 'argtest'), 'rb') as f:
            module = base64.b64encode(f.read()).decode()
        send_message(proc.stdin, 'Module', dict(module=module, module_name='argtest'))
        message = await read_message(proc.stdout)
        assert message[0] == "ModuleResult"
        assert message[1] != {}
        assert message[1]['stdout']
    finally:
        send_message(proc.stdin, 'Shutdown', {})
        await proc.wait()
        os.unlink(ftl_gate)
        clean_up_ftl_cache()
        clean_up_tmp()


@pytest.mark.asyncio
async def test_run_ftl_module():
    os.chdir(HERE)
    ftl_gate = build_ftl_gate()
    proc = await asyncio.create_subprocess_shell(
        ftl_gate,
        stdin=asyncio.subprocess.PIPE,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE)

    try:
        send_message(proc.stdin, 'Hello', {})
        message = await read_message(proc.stdout)
        assert message[0] == "Hello"
        assert message[1] == {}

        with open(find_module(['ftl_modules'], 'argtest'), 'rb') as f:
            module = base64.b64encode(f.read()).decode()
        send_message(proc.stdin, 'FTLModule', dict(module=module, module_name='argtest'))
        message = await read_message(proc.stdout)
        assert message[0] == "FTLModuleResult"
        assert message[1] != {}
        assert message[1]['result']
    finally:
        send_message(proc.stdin, 'Shutdown', {})
        await proc.wait()
        os.unlink(ftl_gate)
        clean_up_tmp()




import asyncio
import asyncssh
import asyncssh.misc
import base64
import json
import os
import io
import sys
import shutil
import tempfile
import uuid
from functools import partial

from .message import send_message_str, read_message
from .gate import build_ftl_gate
from .util import chunk, find_module


async def check_output(cmd, env=None, stdin=None):
    proc = await asyncio.create_subprocess_shell(
        cmd,
        stdin=asyncio.subprocess.PIPE,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.STDOUT,
        env=env)

    stdout, stderr = await proc.communicate(stdin)
    return stdout


async def check_version(conn, interpreter):
    result = await conn.run(f'{interpreter} --version')
    python_version = result.stdout.strip()
    for line in python_version.split('\n'):
        line = line.strip()
        if line.startswith('Python '):
            _, _, version = line.partition(' ')
            major, _, _ = version.split('.')
            if int(major) < 3:
                raise Exception('Python 3 or greater required for interpreter')
        else:
            raise Exception(f'Ensure that non-interactive shells emit no text: {line}')

async def send_gate(gate_builder, conn, tempdir, interpreter):
    ftl_gate = gate_builder(interpreter=interpreter)
    async with conn.start_sftp_client() as sftp:
        await sftp.put(ftl_gate, f'{tempdir}/ftl_gate.pyz')
    result = await conn.run(f'chmod 700 {tempdir}/ftl_gate.pyz', check=True)
    assert result.exit_status == 0


async def open_gate(conn, tempdir):
    process = await conn.create_process(f'{tempdir}/ftl_gate.pyz')
    send_message_str(process.stdin, "Hello", {})
    assert await read_message(process.stdout) == ['Hello', {}]
    return process


async def run_module_through_gate(gate_process, module, module_name):
    #with open(module, 'rb') as f:
    #    module_text = base64.b64encode(f.read()).decode()
    module_text=None
    send_message_str(gate_process.stdin, 'Module', dict(module=module_text, module_name=module_name))
    return await read_message(gate_process.stdout)


async def run_ftl_module_through_gate(gate_process, module, module_name):
    with open(module, 'rb') as f:
        module_text = base64.b64encode(f.read()).decode()
    send_message_str(gate_process.stdin, 'FTLModule', dict(module=module_text, module_name=module_name))
    return await read_message(gate_process.stdout)



def is_new_style_module(module):
    return True


async def run_module_locally(host_name, host, module, module_args):
    tmp = tempfile.mkdtemp()
    tmp_module = os.path.join(tmp, 'module.py')
    shutil.copy(module, tmp_module)
    # TODO: replace hashbang with ansible_python_interpreter
    # TODO: add utf-8 encoding line
    args = os.path.join(tmp, 'args')
    with open(args, 'w') as f:
        f.write(json.dumps(module_args))
    interpreter = host.get('ansible_python_interpreter', '/usr/bin/python')
    if is_new_style_module(module):
        output = await check_output(f'{interpreter} {tmp_module}', stdin=json.dumps(dict(ANSIBLE_MODULE_ARGS=module_args)).encode())
    else:
        output = await check_output(f'{interpreter} {tmp_module} {args}')
    try:
        return host_name, json.loads(output)
    except Exception:
        print(output)
        return host_name, dict(error=output)


async def run_ftl_module_locally(host_name, host, module_path):

    with open(module_path, 'rb') as f:
        module_compiled = compile(f.read(), module_path, 'exec')

    globals = {'__file__': module_path}
    locals = {}

    exec(module_compiled, globals, locals)
    result = await locals['main']()
    return host_name, result


async def remove_item_from_cache(gate_cache):
    if gate_cache is not None and gate_cache:
        item, (conn, gate_process, tempdir) = gate_cache.popitem()
        await close_gate(conn, gate_process, tempdir)
        print('closed gate', item)


async def connect_gate(gate_builder, ssh_host, gate_cache, interpreter):
    while True:
        try:
            conn = await asyncssh.connect(ssh_host)
            await check_version(conn, interpreter)
            tempdir = f'/tmp/ftl-{uuid.uuid4()}'
            result = await conn.run(f'mkdir {tempdir}', check=True)
            result = await conn.run(f'touch {os.path.join(tempdir, "args")}', check=True)
            assert result.exit_status == 0
            await send_gate(gate_builder, conn, tempdir, interpreter)
            gate_process = await open_gate(conn, tempdir)
            return conn, gate_process, tempdir
        except ConnectionResetError:
            print('retry connection')
            await remove_item_from_cache(gate_cache)
            continue
        except asyncssh.misc.ConnectionLost:
            print('retry connection')
            await remove_item_from_cache(gate_cache)
            continue


async def close_gate(conn, gate_process, tempdir):

    try:
        if gate_process is not None:
            send_message_str(gate_process.stdin, "Shutdown", {})
        if gate_process is not None and gate_process.exit_status is not None:
            await gate_process.stderr.read()
    finally:
        conn.close()
    #result = await conn.run(f'rm -rf {tempdir}', check=True)
    #assert result.exit_status == 0


async def run_module_on_host(host_name, host, module, module_args, local_runner, remote_runner,
                             gate_cache, gate_builder):
    if host and host.get('ansible_connection') == 'local':
        return await local_runner(host_name, host, module, module_args)
    else:
        module_name = os.path.basename(module)
        if host and host.get('ansible_host'):
            ssh_host = host.get('ansible_host')
        else:
            ssh_host = host_name
        if host and host.get('ansible_python_interpreter'):
            interpreter = host.get('ansible_python_interpreter')
        else:
            interpreter = sys.executable
        while True:
            try:
                if gate_cache is not None and gate_cache.get(host_name):
                    conn, gate_process, tempdir = gate_cache.get(host_name)
                    del gate_cache[host_name]
                else:
                    conn, gate_process, tempdir = await connect_gate(gate_builder, ssh_host, gate_cache, interpreter)
                try:
                    return host_name, await remote_runner(gate_process, module, module_name)
                finally:
                    if gate_cache is None:
                        await close_gate(conn, gate_process, tempdir)
                    else:
                        gate_cache[host_name] = (conn, gate_process, tempdir)
                break
            except ConnectionResetError:
                print('retry connection')
                #Randomly close a connection in the cache
                await remove_item_from_cache(gate_cache)
                continue

        return host_name, None


def unique_hosts(inventory):

    hosts = {}

    for group_name, group in inventory.items():
        for host_name, host in group.get('hosts').items():
            hosts[host_name] = host

    return hosts


def extract_task_results(tasks):

    results = {}
    for task in tasks:
        host_name, result = task.result()
        results[host_name] = result
    return results


async def _run_module(inventory, module_dirs, module_name, local_runner,
                      remote_runner, gate_cache, modules, dependencies, module_args):

    module = find_module(module_dirs, module_name)

    if module is None:
        raise Exception(f'Module {module_name} not found in {module_dirs}')

    hosts = unique_hosts(inventory)

    gate_builder = partial(build_ftl_gate, modules=modules, module_dirs=module_dirs, dependencies=dependencies)

    all_tasks = []
    # Run the tasks in chunks of 10 to reduce contention for remote connections.
    # This doubles performance at num_hosts=1000 for remote execution
    for c in chunk(list(hosts.items()), 10):
        tasks = []
        for host_name, host in c:
            tasks.append(asyncio.create_task(run_module_on_host(host_name,
                                                                host,
                                                                module,
                                                                module_args,
                                                                local_runner,
                                                                remote_runner,
                                                                gate_cache,
                                                                gate_builder)))
        await asyncio.gather(*tasks)
        for i in extract_task_results(tasks):
            print('ok:', f'[{i}]')

        all_tasks.extend(tasks)

    return extract_task_results(all_tasks)


async def run_module(inventory, module_dirs, module_name, gate_cache=None, modules=None, dependencies=None, module_args=None):
    '''
    Runs a module on all items in an inventory concurrently.
    '''

    return await _run_module(inventory,
                             module_dirs,
                             module_name,
                             run_module_locally,
                             run_module_through_gate,
                             gate_cache,
                             modules,
                             dependencies,
                             module_args)


async def run_ftl_module(inventory, module_dirs, module_name, gate_cache=None, modules=None, dependencies=None):
    '''
    Runs a module on all items in an inventory concurrently.
    '''

    return await _run_module(inventory,
                             module_dirs,
                             module_name,
                             run_ftl_module_locally,
                             run_ftl_module_through_gate,
                             gate_cache,
                             modules,
                             dependencies)




import json


def send_message(writer, msg_type, msg_data):
    message = json.dumps([msg_type, msg_data]).encode()
    #print('{:08x}'.format(len(message)).encode())
    #print(message)
    writer.write('{:08x}'.format(len(message)).encode())
    writer.write(message)


def send_message_str(writer, msg_type, msg_data):
    message = json.dumps([msg_type, msg_data])
    #print('{:08x}'.format(len(message)))
    #print(message)
    writer.write('{:08x}'.format(len(message)))
    writer.write(message)


async def read_message(reader):
    while True:
        length = await reader.read(8)
        try:
            value = await reader.read(int(length, 16))
        except ValueError:
            #print(f'length {length}')
            raise
        return json.loads(value)




import hashlib
import os
import sys
import shutil
import tempfile
import zipapp

from importlib_resources import files
import faster_than_light.ftl_gate
from subprocess import check_output

from .util import ensure_directory, read_module


def build_ftl_gate(modules=None, module_dirs=None, dependencies=None, interpreter=sys.executable):

    cache = ensure_directory('~/.ftl')

    if modules is None:
        modules = []
    if module_dirs is None:
        module_dirs = []
    if dependencies is None:
        dependencies = []

    inputs = []
    inputs.extend(modules)
    inputs.extend(module_dirs)
    inputs.extend(dependencies)
    inputs.extend(interpreter)

    gate_hash = hashlib.sha256("".join(inputs).encode()).hexdigest()

    cached_gate = os.path.join(cache, f'ftl_gate_{gate_hash}.pyz')
    if os.path.exists(cached_gate):
        return cached_gate

    tempdir = tempfile.mkdtemp()
    os.mkdir(os.path.join(tempdir, 'ftl_gate'))
    with open(os.path.join(tempdir, 'ftl_gate', '__main__.py'), 'w') as f:
        f.write(files(faster_than_light.ftl_gate).joinpath('__main__.py').read_text())

    module_dir = os.path.join(tempdir, "ftl_gate", "ftl_gate")
    os.makedirs(module_dir)
    with open(os.path.join(module_dir, "__init__.py"), 'w') as f:
        f.write("")

    # Install modules
    if modules:
        for module in modules:
            with open(os.path.join(module_dir, f'{module}.py'), 'wb') as f:
                f.write(read_module(module_dirs, module))

    # Install dependencies for Gate
    if dependencies:
        requirements = os.path.join(tempdir, 'requirements.txt')
        with open(requirements, 'w') as f:
            f.write("\n".join(dependencies))
        output = check_output([interpreter, '-m', 'pip',
                               'install',
                               '-r', requirements,
                               '--target', os.path.join(tempdir, "ftl_gate")])
        print(output)

    zipapp.create_archive(os.path.join(tempdir, 'ftl_gate'),
                          os.path.join(tempdir, 'ftl_gate.pyz'),
                          interpreter)
    shutil.rmtree(os.path.join(tempdir, 'ftl_gate'))
    shutil.copy(os.path.join(tempdir, 'ftl_gate.pyz'), cached_gate)

    return cached_gate





from faster_than_light.inventory import load_inventory
import os

HERE = os.path.dirname(os.path.abspath(__file__))


def test_inventory():
    os.chdir(HERE)
    inventory = load_inventory('inventory.yml')
    assert inventory
